function [] = apply_mask_programmatic(subj_cell,mask_cell,func_cell,output_dir)
%apply_mask_programmatic applies the masks generated by
%create_mask_programmatic to the functional data in order to restrict
%analysis to just those voxels.
%
%Inputs:
%   subj_cell   =   an Nx1 cell array containing the subject id's for N
%                   subjects
%   mask_cell   =   an Nx1 cell array containing the paths to the masks
%                   generated by create_mask_programmatic
%   func_cell   =   an Nx1 cell array containing the paths to the
%                   functional images for each subject. These will be
%                   masked by the masks from mask_cell
%   output_dir  =   the name of the output directory

if ~exist(output_dir,'dir')
    mkdir(output_dir)
end

subs = size(subj_cell,1);
for ss = 1:subs
    if ~exist([output_dir '\' subj_cell{ss}],'dir')
        mkdir([output_dir '\' subj_cell{ss}])
    end
    fV = spm_vol(func_cell{ss});
    mfV = fV;
    mVol = spm_read_vols(spm_vol(mask_cell{ss}));
    for vv = 1:size(fV,1)
        mfV(vv).fname = [output_dir '\' subj_cell{ss} '\masked_' subj_cell{ss} '.nii'];
        mfV(vv).descript = 'Masked';
        mfVol = spm_read_vols(fV(vv));
        mfVol = mfVol.*mVol;
        spm_write_vol(mfV(vv),mfVol);
    end
    prog = ['Subject ' subj_cell{ss} ' is done.\n'];
    fprintf(prog);
end

end