function create_mask_gui()
%create_mask_gui is the gui for create_mask_programmatic. It also supports
%saving/loading config files from the other guis in this toolbox


%% Argument Management

if nargin == 0
    subject_list = '';
end
 
 %% GUI initialization

f_cmg = figure('Visible','on','MenuBar','none','Color',[0.94,0.94,0.94], ...
    'Units','normalized','OuterPosition',[0.25,0.2,0.5,0.6]);

uicontrol(f_cmg,'Style','text','String','Mask Creator', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0,0.9375,1,0.05]);

p_subj = uipanel(f_cmg,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Subject IDs','FontUnits','normalized','FontSize',0.05, ...
    'Units','normalized','Position',[0.025,0.35,0.296,0.6]);
sub_ids = uicontrol(p_subj,'Style','edit','String',subject_list, ...
    'Max',2,'FontUnits','points','FontSize',12, ...
    'Units','normalized','Position',[0.025,0.175,0.95,0.8],'Callback',@update_subj_list);
uicontrol(p_subj,'Style','pushbutton','String','Import Subject List', ...
    'FontUnits','normalized','FontSize',0.35,'Units','normalized', ...
    'Position',[0.0250,0.025,0.4625,0.10],'Callback',@import_subj_list);
uicontrol(p_subj,'Style','pushbutton','String','Export Subject List', ...
    'FontUnits','normalized','FontSize',0.35,'Units','normalized', ...
    'Position',[0.5125,0.025,0.4625,0.10],'Callback',@export_subj_list);

p_mask = uipanel(f_cmg,'BackgroundColor',[0.9,0.9,0.9], ... 
    'Title','Mask Source Images','FontUnits','normalized', ...
    'FontSize',0.05,'Units','normalized', ...
    'Position',[0.346,0.35,0.308,0.6]);
subj_cell_m = get(sub_ids,'String');
mask_list = uicontrol(p_mask,'Style','listbox','String',subj_cell_m, ...
    'FontUnits','points','FontSize',12,'Enable','on', ...
    'Units','normalized','Position',[0.025,0.175,0.95,0.8], ...
    'Callback',{@get_img_source,'mask'});
uicontrol(p_mask,'Style','pushbutton','String','Reset', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.0250,0.025,0.95,0.10],'Callback',{@reset_list,mask_list});

p_func = uipanel(f_cmg,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Functional Space','FontUnits','normalized', ...
    'FontSize',0.05,'Units','normalized', ...
    'Position',[0.679,0.35,0.296,0.6]);
subj_cell_f = get(sub_ids,'String');
func_list = uicontrol(p_func,'Style','listbox','String',subj_cell_f, ...
    'FontUnits','points','FontSize',12,'Enable','on', ...
    'Units','normalized','Position',[0.025,0.175,0.95,0.8], ...
    'Callback',{@get_img_source,'func'});
uicontrol(p_func,'Style','pushbutton','String','Reset', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.0250,0.025,0.95,0.10],'Callback',{@reset_list,func_list});

uicontrol(f_cmg,'Style','text','String','Masking Threshold:', ...
    'FontUnits','normalized','FontSize',0.625,'Units','normalized', ...
    'Position',[0.166,0.25,0.166,0.04]);
thresh_edit = uicontrol(f_cmg,'Style','edit','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized', ...
    'Position',[0.334,0.25,0.154,0.05],'Callback',@update_thresh);

uicontrol(f_cmg,'Style','text','String','Prefix Label:', ...
    'FontUnits','normalized','FontSize',0.625,'Units','normalized', ...
    'Position',[0.5125,0.25,0.154,0.04]);
prefix_edit = uicontrol(f_cmg,'Style','edit','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized', ...
    'Position',[0.667,0.25,0.166,0.05],'Callback',@update_label);

uicontrol(f_cmg,'Style','text','String','Output Directory:', ...
    'FontUnits','normalized','FontSize',0.625,'Units','normalized', ...
    'Position',[0.05,0.15,0.2,0.04]);
output_edit = uicontrol(f_cmg,'Style','edit','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized','Position',[0.25,0.15,0.5,0.05]);
uicontrol(f_cmg,'Style','pushbutton','String','Browse', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.775,0.15,0.15,0.05],'Callback',@browse);

uicontrol(f_cmg,'Style','pushbutton','String','Save Config', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.15,0.025,0.2,0.075],'Callback',@save_config);
uicontrol(f_cmg,'Style','pushbutton','String','Load Config', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.40,0.025,0.2,0.075],'Callback',@load_config);

uicontrol(f_cmg,'Style','pushbutton','String','Run', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.65,0.025,0.2,0.075],'Callback',@run_mask_creator);

handles = guihandles(f_cmg);
subject_list = get(sub_ids,'String');
if ischar(subject_list)
    subject_list = cellstr(subject_list);
end
handles.subject_list = subject_list;
handles.mask_image_list = get(mask_list,'String');
handles.func_image_list = get(func_list,'String');
handles.threshold = str2double(get(thresh_edit,'String'));
handles.label = get(prefix_edit,'String');
handles.mask_output_dir = get(output_edit,'String');
handles.masked_output_dir = [];
handles.kernel = [];
handles.smooth_output_dir = [];
guidata(f_cmg,handles);

%% Callbacks

    function update_subj_list(hObject,~)
        subj_list = get(hObject,'String');
        if ischar(subj_list)
            subj_list = cellstr(subj_list);
        end
        handles = guidata(f_cmg);
        handles.subject_list = subj_list;
        guidata(f_cmg,handles);
    end

    function import_subj_list(~,~)
	%This function opens a file selector dialog box for choosing a subject list file
	%It also sets the content of the edit and list boxes to the imported subject list
        [fni,pni,~] = uigetfile('*.mat','Import Subject List:');
        if fni ~= 0
            ifile = fullfile(pni,fni);
            list = load(ifile);
            set(sub_ids,'String',list.subj_list);
            set(mask_list,'String',list.subj_list);
            set(func_list,'String',list.subj_list);
        end
    end

    function export_subj_list(~,~)
	%This function opens a file selector dialog box for the user to set the name of a subject list file
	%It then saves the contenet of the edit box as a .mat file
        exp_list = get(sub_ids,'String');
        if ischar(exp_list)
            exp_list = cellstr(exp_list); %#ok<NASGU>
        end
        [fne,pne,~] = uiputfile('Subject_List.mat','Export Subject List:');
        if fne ~= 0
            efile = fullfile(pne,fne);
            save(efile,'exp_list');
        end
    end

    function get_img_source(hObject,~,type)
	%This function is used by both the mask specification and functional space specification list boxes to get the paths to the relevant images
	%It also changes the content of the boxes to the file path
        list_index = get(hObject,'Value');
        img_cell = get(hObject,'String');
        subj_list = get(sub_ids,'String');
        if ~isempty(list_index)
            if strcmp(type,'mask')
                dstr = ['Choose the Mask Source image for subject ' subj_list{list_index}];
            elseif strcmp(type,'func')
                dstr = ['Choose the Functional image for subject ' subj_list{list_index}];
            end
            [fns,pns,~] = uigetfile('*.nii',dstr);
            if fns ~= 0
                sfile = fullfile(pns,fns);
                img_cell{list_index} = sfile;
                set(hObject,'String',img_cell);
            end
        end
    end

    function reset_list(~,~,list)
	%This function resets the content of the list boxes
        reset_string = get(sub_ids,'String');
        if ~isempty(reset_string)
            if ischar(reset_string)
                reset_string = cellstr(reset_string); 
            end
            set(list,'Value',1);
            set(list,'String',reset_string);
        end
    end

    function update_thresh(hObject,~)
        handles = guidata(f_cmg);
        handles.threshold = str2double(get(hObject,'String'));
        guidata(f_cmg,handles)
    end

    function update_label(hObject,~)
        handles = guidata(f_cmg);
        handles.label = get(hObject,'String');
        guidata(f_cmg,handles);
    end

    function browse(~,~)
	%This function opens a file selector dialog box to select or create a new folder to store the masks in
       folder_name = uigetdir(pwd,'Choose or Create the output folder: ');
       if folder_name ~= 0
           set(output_edit,'String',folder_name);
           handles = guidata(f_cmg);
           handles.mask_output_dir = folder_name;
           guidata(f_cmg,handles);
       end
    end

    function save_config(~,~)
	%This function saves every relevant variable to make a configuration file
       [fnsc,pnsc,~] = uiputfile('create_mask_config.mat','Save Configuration File:');
       if fnsc ~= 0
            handles = guidata(f_cmg);
            subj_cell = handles.subject_list;
            mask_cell = handles.mask_image_list;
            func_cell = handles.func_image_list;
            thresh = handles.threshold;
            label = handles.label;
            mask_output_dir = handles.mask_output_dir;
            func_output_dir = handles.masked_output_dir;
            kernel = handles.kernel;
            smooth_output_dir = handles.smooth_output_dir;
            filesc = fullfile(pnsc,fnsc);
            save(filesc,'subj_cell','mask_cell','func_cell','thresh','label','mask_output_dir','func_output_dir','kernel','smooth_output_dir');
       end
    end

    function load_config(~,~)
	%This function loads a configuration file
        [fnlc,pnlc,~] = uigetfile('*.mat','Load Configuration File:');
        if fnlc ~= 0
            filelc = fullfile(pnlc,fnlc);
            config = load(filelc);
            handles = guidata(f_cmg);
            handles.subject_list = config.subj_cell;
            set(sub_ids,'String',config.subj_cell);
            handles.mask_image_list = config.mask_cell;
            set(mask_list,'String',config.mask_cell);
            handles.func_image_list = config.func_cell;
            set(func_list,'String',config.func_cell);
            handles.threshol = config.thresh;
            set(thresh_edit,'String',config.thresh);
            handles.label = config.label;
            set(prefix_edit,'String',config.label);
            handles.mask_output_dir = config.mask_output_dir;
            set(mask_o_edit,'String',config.mask_output_dir);
            handles.masked_output_dir = config.func_output_dir;
            handles.kernel = config.kernel;
            handles.smooth_output_dir = config.smooth_output_dir;
            guidata(f_cmg,handles)
        end
    end

    function run_mask_creator(~,~)
	%This function runs the actual functions
        handles = guidata(f_cmg);
        subj_cell = handles.subject_list;
        mask_cell = handles.mask_image_list;
        func_cell = handles.func_image_list;
        thresh = handles.threshold;
        if ischar(thresh)
            thresh = strdouble(thresh);
        end
        label = handles.label;
        mask_output_dir = handles.mask_output_dir;
        func_output_dir = handles.masked_output_dir;  
        kernel = handles.kernel;
        smooth_output_dir = handles.smooth_output_dir;
        
        c = clock; c_str = [num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_igma_config_autosave.mat'];
        save(c_str,'subj_cell','mask_cell','func_cell','thresh','label','mask_output_dir','func_output_dir','kernel','smooth_output_dir');
        
        create_mask_programmatic(subj_cell,mask_cell,func_cell,thresh,label,mask_output_dir)
        pause(0.5)
        d = [mask_output_dir '\*.nii'];
        d = dir(d);
        mask_files = struct2cell(d);
        mask_files(2:end,:) = [];
        mask_files = mask_files';
        for ii = 1:size(mask_files,1)
          mask_files{ii} = [mask_output_dir '\' mask_files{ii}];
        end
        overlap_map_programmatic(mask_files,pwd)
        close(f_cmg)
    end

end