function [] = create_mask_programmatic(subj_cell,mask_cell,func_cell,threshold,prefix,output_dir)
%create_mask_programmatic generates masks for each subject based on a
%specified threshold. It was originally designed to process segmented
%tissue probability maps generated by spm12's segmentation function, though
%it can generate masks for any given image as long as an appropriate
%threshold is specified
%
%Inputs:
%   subj_cell   =   an Nx1 cell array containing the subject id's for N
%                   subjects
%   mask_cell   =   an Nx1 cell array containing the paths to the images
%                   that the masks will be derived from
%   func_cell   =   an Nx1 cell array containing the paths to the
%                   functional images for each subject. The generated masks
%                   will be resliced to be in the functional images' space
%   threshold   =   the masking threshold for the images from mask_cell
%   prefix      =   a prefix that will be applied to the beginning of the
%                   mask's filenames
%   output_dir  =   the name of the output directory

if ~exist(output_dir,'dir')
	mkdir(output_dir)
end

subs = size(subj_cell,1);
for ss = 1:subs
    sV = spm_vol(mask_cell{ss});
    sVol = spm_read_vols(sV);
    
    sVol(sVol > threshold) = 1;
    sVol(sVol <= threshold) = 0;
    sV.fname = [output_dir '\' prefix '_' subj_cell{ss} '.nii'];
    spm_write_vol(sV,sVol);
    
    fV = spm_vol(func_cell{ss});
    fV = fV(1);
    fVol = spm_read_vols(fV);
    fV.fname = [output_dir '\FirstVol_' subj_cell{ss} '.nii'];
    spm_write_vol(fV,fVol);
    
    P = {fV.fname;sV.fname};
    flags = struct('mask',true,'mean',false,'interp',1,'which',1,'wrap',[1 1 0],'prefix','r');
    spm_reslice(P,flags);
    
    delete(sV.fname,fV.fname);
    prog = ['Subject ' subj_cell{ss} ' is done.\n'];
    fprintf(prog)
end

end