function mask_and_smooth_gui()
%mask_and_smooth_gui pulls together create_mask_programmatic,
%apply_mask_programmatic, and smooth_programmatic, since many of their
%inputs are the same or can be inferred from the previous step

%% Argument Management

if nargin == 0
    subject_list = '';
end

%% GUI Initialization

f_ccg = figure('Visible','on','MenuBar','none','Color',[0.94,0.94,0.94], ...
    'Units','normalized','OuterPosition',[0.25,0.2,0.5,0.6]);

uicontrol(f_ccg,'Style','text','String','Post-Normalization Masking', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0,0.9375,1,0.05]);

p_subj = uipanel(f_ccg,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Subject IDs','FontUnits','normalized','FontSize',0.05, ...
    'Units','normalized','Position',[0.025,0.35,0.296,0.6]);
sub_ids = uicontrol(p_subj,'Style','edit','String',subject_list, ...
    'Max',2,'FontUnits','points','FontSize',12, ...
    'Units','normalized','Position',[0.025,0.175,0.95,0.8], ...
    'Callback',@update_subj_list);
uicontrol(p_subj,'Style','pushbutton','String','Import Subject List', ...
    'FontUnits','normalized','FontSize',0.35,'Units','normalized', ...
    'Position',[0.0250,0.025,0.4625,0.10],'Callback',@import_subj_list);
uicontrol(p_subj,'Style','pushbutton','String','Export Subject List', ...
    'FontUnits','normalized','FontSize',0.35,'Units','normalized', ...
    'Position',[0.5125,0.025,0.4625,0.10],'Callback',@export_subj_list);

p_mask = uipanel(f_ccg,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Mask Source Images','FontUnits','normalized', ...
    'FontSize',0.05,'Units','normalized', ...
    'Position',[0.346,0.35,0.308,0.6]);
subj_cell_m = get(sub_ids,'String');
mask_list = uicontrol(p_mask,'Style','listbox','String',subj_cell_m, ...
    'FontUnits','points','FontSize',12,'Enable','on', ...
    'Units','normalized','Position',[0.025,0.175,0.95,0.8], ...
    'Callback',{@get_img_source,'mask'});
uicontrol(p_mask,'Style','pushbutton','String','Reset', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.0250,0.025,0.95,0.10],'Callback',{@reset_list,mask_list,'mask'});

p_func = uipanel(f_ccg,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Functional Images','FontUnits','normalized', ...
    'FontSize',0.05,'Units','normalized', ...
    'Position',[0.679,0.35,0.296,0.6]);
subj_cell_f = get(sub_ids,'String');
func_list = uicontrol(p_func,'Style','listbox','String',subj_cell_f, ...
    'FontUnits','points','FontSize',12,'Enable','on', ...
    'Units','normalized','Position',[0.025,0.175,0.95,0.8], ...
    'Callback',{@get_img_source,'func'});
uicontrol(p_func,'Style','pushbutton','String','Reset', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.0250,0.025,0.95,0.10],'Callback',{@reset_list,func_list,'func'});

uicontrol(f_ccg,'Style','text','String','Masking Threshold:', ...
    'FontUnits','normalized','FontSize',0.625,'Units','normalized', ...
    'Position',[0.025,0.275,0.16,0.04]);
thresh_edit = uicontrol(f_ccg,'Style','edit','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized', ...
    'Position',[0.19,0.275,0.0475,0.05],'Callback',@update_thresh);

uicontrol(f_ccg,'Style','text','String','Label:', ...
    'FontUnits','normalized','FontSize',0.625,'Units','normalized', ...
    'Position',[0.2625,0.275,0.05,0.04]);
plabel_edit = uicontrol(f_ccg,'Style','edit','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized', ...
    'Position',[0.3175,0.275,0.17,0.05],'Callback',@update_label);

uicontrol(f_ccg,'Style','text','String','Mask Ouput Directory:', ...
    'FontUnits','normalized','FontSize',0.625,'Units','normalized', ...
    'Position',[0.5125,0.275,0.175,0.04]);
mask_o_edit = uicontrol(f_ccg,'Style','edit','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized', ...
    'Position',[0.6925,0.275,0.2075,0.05]);
uicontrol(f_ccg,'Style','pushbutton','String','Browse', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.905,0.275,0.07,0.05],'Callback',{@browse,mask_o_edit,'mask_out'});

uicontrol(f_ccg,'Style','text', ...
    'String','Masked Functional Image Output Directory:', ...
    'FontUnits','normalized','FontSize',0.625,'Units','normalized', ...
    'Position',[0.025,0.2,0.34,0.04]);
masked_o_edit = uicontrol(f_ccg,'Style','edit','FontUnits','normalized',...
    'FontSize',0.5,'Units','normalized', ...
    'Position',[0.37,0.2,0.53,0.05]);
uicontrol(f_ccg,'Style','pushbutton','String','Browse', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.905,0.2,0.07,0.05],'Callback',{@browse,masked_o_edit,'masked_out'});

uicontrol(f_ccg,'Style','text','String','Smoothing Kernel:', ...
    'FontUnits','normalized','FontSize',0.55,'Units','normalized', ...
    'Position',[0.025,0.125,0.13,0.04]);
smooth_edit = uicontrol(f_ccg,'Style','edit','String','[8 8 8]', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.16,0.125,0.0775,0.05],'Callback',@update_kernel);

uicontrol(f_ccg,'Style','text','String','Smoothing Output Directory:', ...
    'FontUnits','normalized','FontSize',0.45,'Units','normalized', ...
    'Position',[0.2625,0.125,0.255,0.05]);
smooth_o_edit = uicontrol(f_ccg,'Style','edit','FontUnits','normalized',...
    'FontSize',0.5,'Units','normalized', ...
    'Position',[0.5225,0.125,0.3775,0.05]);
uicontrol(f_ccg,'Style','pushbutton','String','Browse', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.905,0.125,0.07,0.05],'Callback',{@browse,smooth_o_edit,'smooth_out'});

uicontrol(f_ccg,'Style','pushbutton','String','Save Config', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.15,0.025,0.2,0.075],'Callback',@save_config);

uicontrol(f_ccg,'Style','pushbutton','String','Load Config', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ... 
    'Position',[0.4,0.025,0.2,0.075],'Callback',@load_config);

uicontrol(f_ccg,'Style','pushbutton','String','Run', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.65,0.025,0.2,0.075],'Callback',@run_post_normalize);

handles = guihandles(f_ccg);
subject_list = get(sub_ids,'String');
if ischar(subject_list)
    subject_list = cellstr(subject_list);
end
handles.subject_list = subject_list;
handles.mask_image_list = get(mask_list,'String');
handles.func_image_list = get(func_list,'String');
handles.threshold = str2double(get(thresh_edit,'String'));
handles.label = get(plabel_edit,'String');
handles.mask_output_dir = get(mask_o_edit,'String');
handles.masked_output_dir = get(masked_o_edit,'String');
handles.kernel = get(smooth_edit,'String');
handles.smooth_output_dir = get(smooth_o_edit,'String');
guidata(f_ccg,handles);

%% Callbacks

    function update_subj_list(hObject,~)
        subj_list = get(hObject,'String');
        if ischar(subj_list)
            subj_list = cellstr(subj_list);
        end
        handles = guidata(f_ccg);
        handles.subject_list = subj_list;
        guidata(f_ccg,handles);
    end

    function import_subj_list(~,~)
        [fni,pni,~] = uigetfile('*/mat','Import Subject List:');
        if fni ~=0
            ifile = fullfile(pni,fni);
            list = load(ifile);
            handles = guidata(f_ccg);
            handles.subject_list = list.subj_list;
            guidata(f_ccg,handles);
            set(sub_ids,'String',list.subj_list);
            set(mask_list,'String',list.subj_list);
            set(func_list,'String',list.subj_list);
        end
    end

    function export_subj_list(~,~)
        [subj_list] = get(sub_ids,'String');
        if ischar(subj_list)
            subj_list = cellstr(subj_list);
        end
        handles = guidata(f_ccg);
        handles.subject_list = subj_list;
        guidata(f_ccg,handles);
        [fne,pne,~] = uiputfile('Subject_List.mat','Export Subject List:');
        if fne ~= 0
            efile = fullfile(pne,fne);
            save(efile,'subj_list');
        end
    end

    function get_img_source(hObject,~,type)
        list_index = get(hObject,'Value');
        img_cell = get(hObject,'String');
        subj_list = get(sub_ids,'String');
        handles = guidata(f_ccg);
        if ~isempty(list_index) && ~isempty(subj_list)
            if ischar(subj_list)
                subj_list = cellstr(subj_list);
            end
            handles.subject_list = subj_list;
            if strcmp(type,'mask')
                dstr = ['Choose the Mask Source image for subject ' subj_list{list_index}];
            elseif strcmp(type,'func')
                dstr = ['Choose the Functional image for subject ' subj_list{list_index}];
            end
            [fns,pns,~] = uigetfile('*.nii',dstr);
            if fns ~= 0
                sfile = fullfile(pns,fns);
                [~,name,ext] = fileparts(sfile);
                if strcmp(type,'mask')
                    handles.mask_image_list{list_index} = sfile;
                elseif strcmp(type,'func')
                    handles.func_image_list{list_index} = sfile;
                end
                guidata(f_ccg,handles);
                img_cell{list_index} = [name ext];
                set(hObject,'String',img_cell);
            end
        elseif isempty(list_index) && ~isempty(subj_list)
            if ischar(subj_list)
                subj_list = cellstr(subj_list);
            end
            set(hObject,'Value',1);
            set(hObject,'String',subj_list);
        end
    end

    function reset_list(~,~,list,type)
        reset_string = get(sub_ids,'String');
        handles = guidata(f_ccg);
        if ~isempty(reset_string)
            if ischar(reset_string)
                reset_string = cellstr(reset_string);
            end
            set(list,'Value',1);
            set(list,'String',reset_string);
            if strcmp(type,'mask')
                handles.mask_image_list = get(mask_list,'String');
            elseif strcmp(type,'func')
                handles.func_image_list = get(func_list,'String');
            end
            guidata(f_ccg,handles);
        end
    end

    function update_thresh(hObject,~)
        handles = guidata(f_ccg);
        handles.threshold = str2double(get(hObject,'String'));
        guidata(f_ccg,handles);
    end

    function update_label(hObject,~)
        handles = guidata(f_ccg);
        handles.label = get(hObject,'String');
        guidata(f_ccg,handles);
    end

    function update_kernel(hObject,~)
        handles = guidata(f_ccg);
        handles.kernel = get(hObject,'String');
        guidata(f_ccg,handles)
    end

    function browse(~,~,edit_box,type)
        folder_name = uigetdir(pwd,'Choose or Create the output folder: ');
        if folder_name ~= 0
            set(edit_box,'String',folder_name)
        end
        handles = guidata(f_ccg);
        if strcmp(type,'mask_out')
            handles.mask_output_dir = get(edit_box,'String');
        elseif strcmp(type,'masked_out')
            handles.masked_output_dir = get(edit_box,'String');
        elseif strcmp(type,'smooth_out')
            handles.smooth_output_dir = get(edit_box,'String');
        end
        guidata(f_ccg,handles);
    end

    function save_config(~,~)
        [fnsc,pnsc,~] = uiputfile('master_config.mat','Save Configuration File:');
        if fnsc ~= 0
            handles = guidata(f_ccg);
            subj_cell = handles.subject_list;
            mask_cell = handles.mask_image_list;
            func_cell = handles.func_image_list;
            thresh = handles.threshold;
            label = handles.label;
            mask_output_dir = handles.mask_output_dir;
            func_output_dir = handles.masked_output_dir;
            kernel = handles.kernel;
            smooth_output_dir = handles.smooth_output_dir;
            filesc = fullfile(pnsc,fnsc);
            save(filesc,'subj_cell','mask_cell','func_cell','thresh','label','mask_output_dir','func_output_dir','kernel','smooth_output_dir');
        end
    end

    function load_config(~,~)
        [fnlc,pnlc,~] = uigetfile('*.mat','Load Configuration File:');
        if fnlc ~= 0
           filelc = fullfile(pnlc,fnlc);
           config = load(filelc);
           handles = guidata(f_ccg);
           handles.subject_list = config.subj_cell;
           set(sub_ids,'String',config.subj_cell);
           handles.mask_image_list = config.mask_cell;
           set(mask_list,'String',config.mask_cell);
           handles.func_image_list = config.func_cell;
           set(func_list,'String',config.func_cell);
           handles.threshold = config.thresh;
           set(thresh_edit,'String',config.thresh);
           handles.label = config.label;
           set(plabel_edit,'String',config.label);
           handles.mask_output_dir = config.mask_output_dir;
           set(mask_o_edit,'String',config.mask_output_dir);
           handles.masked_output_dir = config.func_output_dir;
           set(masked_o_edit,'String',config.func_output_dir);
           handles.kernel = config.kernel;
           set(smooth_edit,'String',config.kernel);
           handles.smooth_output_dir = config.smooth_output_dir;
           set(smooth_o_edit,'String',config.smooth_output_dir);
           guidata(f_ccg,handles)
        end
    end

    function run_post_normalize(~,~)
        
        handles = guidata(f_ccg);
        subj_cell = handles.subject_list;
        mask_cell = handles.mask_image_list;
        func_cell = handles.func_image_list;
        thresh = handles.threshold;
        if ischar(thresh)
            thresh = str2double(thresh);
        end
        label = handles.label;
        mask_output_dir = handles.mask_output_dir;
        func_output_dir = handles.masked_output_dir;  
        kernel = handles.kernel;
        smooth_output_dir = handles.smooth_output_dir;
        
        c = clock; c_str = [num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_igma_config_autosave.mat'];
        save(c_str,'subj_cell','mask_cell','func_cell','thresh','label','mask_output_dir','func_output_dir','kernel','smooth_output_dir');
        
        fprintf('Creating Masks ... ... ... ... ...')
        create_mask_programmatic(subj_cell,mask_cell,func_cell,thresh,label,mask_output_dir);
        prnt_str1 = ['Done.\nMasks written to ' formatstr(mask_output_dir) '\n'];
        fprintf(prnt_str1)
        
        pause(0.5)
        fprintf('Creating Overlap Map ... ... ... ... ...\n')
        d = [mask_output_dir '\*.nii'];
        d = dir(d);
        mask_files = struct2cell(d);
        mask_files(2:end,:) = [];
        mask_files = mask_files';
        for ii = 1:size(mask_files,1)
            mask_files{ii} = [mask_output_dir '\' mask_files{ii}];
        end
        overlap_map_programmatic(mask_files,mask_output_dir)
        fprintf('Done\n')
        
        clear mask_cell
        if exist('mask_path_cell','var')
            mask_cell = mask_path_cell;
        else
            mask_cell = cell(size(subj_cell));
            for ii = 1:size(subj_cell,1)
                mask_cell{ii,1} = [mask_output_dir '\r' label '_' subj_cell{ii} '.nii'];
            end
        end
        fprintf('Applying Masks ... ... ... ... ...\n')
        apply_mask_programmatic(subj_cell,mask_cell,func_cell,func_output_dir);
        prnt_str2 = ['Done.\nMasked Functional Images written to ' formatstr(func_output_dir) '\n'];
        fprintf(prnt_str2)
        
        clear func_cell
        if exist('masked_normed_imgs','var')
            func_cell = masked_normed_imgs;
        else
            func_cell = cell(size(subj_cell));
            for ii = 1:size(subj_cell,1)
                masked_normed_output_dir = handles.masked_output_dir;
                func_cell{ii,1} = [masked_normed_output_dir '\' subj_cell{ii} '\masked_' subj_cell{ii} '.nii'];
            end
        end
        fprintf('Smoothing ... ... ... ... ...\n');
        smooth_programmatic(subj_cell,mask_cell,func_cell,kernel,smooth_output_dir);
        prnt_str3 = ['Done.\nSmoothed, Masked Functional Images written to ' formatstr(smooth_output_dir) '\n'];
        fprintf(prnt_str3)
    end

end