function overlap_map_gui()
%overlap_map_gui is the gui for overlap_map_programmatic

if nargin == 0
    mask_cell = {};
end

%% GUI Initialization

f_omg = figure('Visible','on','MenuBar','none','Color',[0.94,0.94,0.94], ...
    'Units','normalized','OuterPosition',[0.25,0.2,0.25,0.6]);
uicontrol(f_omg,'Style','text','String','Generate Overlap Map', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0,0.9375,1,0.05]);
uicontrol(f_omg,'Style','text', ...
    'String','Choose the binary masks to merge:', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.025,0.8875,0.6,0.05]);
p_masks = uipanel(f_omg,'BackgroundColor',[0.9,0.9,0.9], ...
    'Units','normalized','Position',[0.025,0.35,0.95,0.525]);
mask_list = uicontrol(p_masks,'Style','listbox', ...
    'String',mask_cell,'FontUnits','points','FontSize',12, ...
    'Enable','on','Units','normalized', ...
    'Position',[0.025,0.15,0.95,0.825]);
uicontrol(p_masks,'Style','pushbutton','String','Add Folder', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.025,0.025,(0.95/3),0.1],'Callback',@add_folder);
uicontrol(p_masks,'Style','pushbutton','String','Add Image', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.025+(0.95/3),0.025,(0.95/3),0.1],'Callback',@add_image);
uicontrol(p_masks,'Style','pushbutton','String','Remove', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.025+2*(0.95/3),0.025,(0.95/3),0.1],'Callback',@remove);
uicontrol(f_omg,'Style','text','String','Choose an Output Directory:', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.025,0.25,0.6,0.05])
output_edit = uicontrol(f_omg,'Style','edit','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized','Position',[0.05,0.2,0.73,0.05]);
uicontrol(f_omg,'Style','pushbutton','String','Browse', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.8,0.2,0.15,0.05],'Callback',{@browse,output_edit});
uicontrol(f_omg,'Style','pushbutton','String','Save Config', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.05,0.05,0.3,0.075],'Callback',@save_config);
uicontrol(f_omg,'Style','pushbutton','String','Load Config', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.35,0.05,0.3,0.075],'Callback',@load_config);
uicontrol(f_omg,'Style','pushbutton','String','Run', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.65,0.05,0.3,0.075],'Callback',@run_overlap_map);
handles = guihandles(f_omg);
handles.mask_cell = mask_cell;
handles.output_dir = get(output_edit,'String');
guidata(f_omg,handles)

%% Callbacks

    function add_folder(~,~)
        folder_name = uigetdir(pwd,'Choose the folder with the masks you want to merge in it:');
        d = dir([folder_name '\*.nii']);
        d = struct2cell(d);
        d(2:end,:) = [];
        d = d';
        for dd = 1:size(d,1)
            d{dd} = [folder_name '\' d{dd}];
        end
        handles = guidata(f_omg);
        mask_cell = handles.mask_cell;
        mask_cell = [mask_cell;d];
        handles.mask_cell = mask_cell;
        guidata(f_omg,handles);
        set(mask_list,'Value',1);
        set(mask_list,'String',mask_cell);
    end

    function add_image(~,~)
        [fni,pni,~] = uigetfile('*.nii','Choose a file:');
        if fni ~= 0
            image = fullfile(pni,fni);
            handles = guidata(f_omg);
            mask_cell = handles.mask_cell;
            mask_cell = [mask_cell;image];
            handles.mask_cell = mask_cell;
            guidata(f_omg,handles);
            set(mask_list,'String',mask_cell);
        end
    end

    function remove(~,~)
        img_index = get(mask_list,'Value');
        handles = guidata(f_omg);
        mask_cell = handles.mask_cell;
        mask_cell(img_index) = [];
        if isempty(mask_cell)
            mask_cell = {''};
        end
        handles.mask_cell = mask_cell;
        guidata(f_omg,handles);
        set(mask_list,'String',mask_cell);
        set(mask_list,'Value',1);
    end

    function browse(~,~,edit_box)
        folder_name = uigetdir(pwd,'Choose or Create the output folder: ');
        if folder_name ~= 0
            set(edit_box,'String',folder_name)
            handles = guidata(f_omg);
            handles.output_dir = folder_name;
            guidata(f_omg,handles);
        end
    end

    function save_config(~,~)
        [fnsc,pnsc,~] = uiputfile('overlap_map_config.mat','Save Configuration File:');
        if fnsc ~= 0
            handles = guidata(f_omg);
            mask_paths = handles.mask_cell;
            output_dir = handles.output_dir;
            filesc = fullfile(pnsc,fnsc);
            save(filesc,'mask_paths','output_dir');
        end
    end

    function load_config(~,~)
        [fnlc,pnlc,~] = uigetfile('*.mat','Load Configuration File:');
        if fnlc ~= 0
            filelc = fullfile(pnlc,fnlc);
            config = load(filelc);
            handles = guidata(f_omg);
            handles.mask_cell = config.mask_paths;
            set(mask_list,'String',config.mask_paths);
            handles.output_dir = config.output_dir;
            set(output_edit,'String',config.output_dir);
            guidata(f_omg,handles);
        end
    end

    function run_overlap_map(~,~)
        handles = guidata(f_omg);
        mask_paths = handles.mask_cell;
        output_dir = handles.output_dir;
        
        c = clock; c_str = [num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_overlap_map_config_autosave.mat'];
        save(c_str,'mask_paths','output_dir');
        
        output_dir = [output_dir '\'];
        prog_str = 'Constructing Overlap Map ... ... ...';
        fprintf(prog_str)
        overlap_map_programmatic(mask_paths,output_dir);
        done_str = ['Done.\nResults written to ' formatstr(output_dir) '\n'];
        fprintf(done_str)
        close(f_omg)
    end

end