function [] = overlap_map_programmatic(mask_cell,output_dir)
%overlap_map_programmatic generates a brain map that shows the number of
%subjects who have valid data at each voxel.
%
%Inputs:
%   mask_cell   =   an Nx1 cell array containing the paths to binary mask
%                   images for N subjects. The binary masks should have a 1
%                   to include voxels with valid data and 0 to exclude
%                   voxels with invalid data
%   output_dir  =   the name of the output directory

if ~exist(output_dir,'dir')
    mkdir(output_dir)
end

subs = size(mask_cell,1);
for ss = 1:subs
    if ss == 1
        sV = spm_vol(mask_cell{ss});
        sVol = spm_read_vols(sV);
    else
        sVol = cat(4,sVol,spm_read_vols(spm_vol(mask_cell{ss})));
    end
end
sVol = sum(sVol,4);
sV.fname = [output_dir '\Overlap_map_' num2str(subs) '_subjects.nii'];
sV.descrip = ['Number of Subjects with data at each voxel. Max = ' num2str(subs)];
sV = rmfield(sV,'pinfo');
sV.dt(1) = 2;
spm_write_vol(sV,sVol);

end

