function [xmod,ymod] = rm_empty_subj(x,y,nSubjs)
%rm_empty_sbuj removes entries from vectors until their size matches
%nSubjs. This is called by oneSampleT_test_custom.m in order to remove
%subjects that do not have valid signal at a given voxel from analysis at
%that voxel.
%
%Inputs:
%   x       =   A vector that is the same size as y. oneSampleT_test_custom
%               passes the vector of ones necessary for linear regression.
%   y       =   A vector that is the same size as x. oneSampleT_test_custom
%               passes the vector that contains the voxel data for each 
%               subject
%   nSubjs  =   The number of valid subjects at this voxel.
%Outputs:
%   xmod    =   The reduced form of the x vector
%   ymod    =   The reduced form of the y vector

% Note to Arjuna: I had to modify this script, in the case that there were
% no zero values in y, find(~y) would return an empty matrix which would
% then cause the while loop to abort. I added an if loop to check on the
% state of y before entering the loop.

if sum(~y) ~= 0

    while length(y) > nSubjs
         ind = find(~y);
         ind = ind(1);
         y(ind) = [];
         x(ind) = [];
    end
    
end

xmod = x;
ymod = y;

end