function smooth_gui()
%smooth_gui is the gui from smooth_programmatic. It also supports
%saving/loading config files from the other guis in this toolbox


%% Argument Management

if nargin == 0
    subject_list = '';
end

%% GUI initialization

f_sg = figure('Visible','on','Color',[0.94,0.94,0.94], ...
    'Units','normalized','OuterPosition',[0.25,0.2,0.5,0.6]);

uicontrol(f_sg,'Style','text','String','Smoothing', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0,0.9375,1,0.05]);

p_subj = uipanel(f_sg,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Subject IDs','FontUnits','normalized','FontSize',0.04, ...
    'Units','normalized','Position',[0.025,0.25,0.296,0.7]);
sub_ids = uicontrol(p_subj,'Style','edit','String',subject_list, ...
    'Max',2,'FontUnits','points','FontSize',12,'Units','normalized', ...
    'Position',[0.025,0.125,0.95,0.85],'Callback',@update_subj_list);
uicontrol(p_subj,'Style','pushbutton','String','Import Subject List', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.0250,0.025,0.4625,0.075],'Callback',@import_subj_list);
uicontrol(p_subj,'Style','pushbutton','String','Export Subject List', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.5125,0.025,0.4625,0.075],'Callback',@export_subj_list);

p_mask = uipanel(f_sg,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Mask Images','FontUnits','normalized','FontSize',0.04, ...
    'Units','normalized','Position',[0.346,0.250,0.308,0.7]);
subj_cell_m = get(sub_ids,'String');
mask_list = uicontrol(p_mask,'Style','listbox','String',subj_cell_m, ...
    'FontUnits','points','FontSize',12,'Enable','on', ...
    'Units','normalized','Position',[0.025,0.125,0.95,0.85], ...
    'Callback',{@get_img_source,'mask'});
uicontrol(p_mask,'Style','pushbutton','String','Reset', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.025,0.025,0.95,0.075],'Callback',{@reset_list,mask_list});

p_func = uipanel(f_sg,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Functional Images','FontUnits','normalized', ...
    'FontSize',0.04,'Units','normalized', ...
    'Position',[0.679,0.25,0.296,0.7]);
subj_cell_f = get(sub_ids,'String');
func_list = uicontrol(p_func,'Style','listbox','String',subj_cell_f, ...
    'FontUnits','points','FontSize',12,'Enable','on', ...
    'Units','normalized','Position',[0.025,0.125,0.95,0.85], ...
    'Callback',{@get_img_source,'func'});
uicontrol(p_func,'Style','pushbutton','String','Reset', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.0250,0.025,0.95,0.075],'Callback',{@reset_list,func_list});

uicontrol(f_sg,'Style','text','String','Smoothing Kernel (3D):', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.2375,0.175,0.25,0.05]);
smooth_edit = uicontrol(f_sg,'Style','edit','String','[ ]','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized', ...
    'Position',[0.5125,0.175,0.25,0.05],'Callback',@update_kernel);

uicontrol(f_sg,'Style','text','String','Output Directory:', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.05,0.1125,0.2,0.05]);
output_edit = uicontrol(f_sg,'Style','edit','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized','Position',[0.25,0.1125,0.5,0.05]);
uicontrol(f_sg,'Style','pushbutton','String','Browse', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.775,0.1125,0.2,0.05],'Callback',@browse);

uicontrol(f_sg,'Style','pushbutton','String','Save Config', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.15,0.025,0.2,0.075],'Callback',@save_config);
uicontrol(f_sg,'Style','pushbutton','String','Load Config', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.40,0.025,0.2,0.075],'Callback',@load_config);
uicontrol(f_sg,'Style','pushbutton','String','Run', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.65,0.025,0.2,0.075],'Callback',@run_smoother);

handles = guihandles(f_sg);
subject_list = get(sub_ids,'String');
if ischar(subject_list)
    subject_list = cellstr(subject_list);
end
handles.subject_list = subject_list;
handles.mask_image_list = get(mask_list,'String');
handles.func_image_list = get(func_list,'String');
handles.threshold = [];
handles.label = [];
handles.mask_output_dir = [];
handles.masked_output_dir = [];
handles.kernel = get(smooth_edit,'String');
handles.smooth_output_dir = get(output_edit,'String');
guidata(f_sg,handles);

%% Callbacks

    function update_subj_list(hObject,~)
        subj_list = get(hObject,'String');
        if ischar(subj_list)
            subj_list = cellstr(subj_list);
        end
        handles = guidata(f_sg);
        handles.subject_list = subj_list;
        guidata(f_sg,handles);
    end

    function import_subj_list(~,~)
	%This function opens a file selector dialog box for choosing a subject list file
	%It also sets the content of the edit and list boxes to the imported subject list
        [fni,pni,~] = uigetfile('*.mat','Import Subject List:');
        if fni ~= 0
            ifile = fullfile(pni,fni);
            list = load(ifile);
            set(sub_ids,'String',list.subj_list);
            set(mask_list,'String',list.subj_list);
            set(func_list,'String',list.subj_list);
        end
    end

    function export_subj_list(~,~)
	%This function opens a file selector dialog box for the user to set the name of a subject list file
	%It then saves the contenet of the edit box as a .mat file
        exp_list = get(sub_ids,'String');
        if ischar(exp_list)
            exp_list = cellstr(exp_list); %#ok<NASGU>
        end
        [fne,pne,~] = uiputfile('Subject_List.mat','Export Subject List:');
        if fne ~= 0
            efile = fullfile(pne,fne);
            save(efile,'exp_list');
        end
    end

    function get_img_source(hObject,~,type)
	%This function is used by both the mask specification and functional space specification list boxes to get the paths to the relevant images
	%It also changes the content of the boxes to the file path
        list_index = get(hObject,'Value');
        img_cell = get(hObject,'String');
        if ~isempty(list_index)
            if strcmp(type,'mask')
                dstr = ['Choose the Mask Source image for subject ' img_cell{list_index}];
            elseif strcmp(type,'func')
                dstr = ['Choose the Functional image for subject ' img_cell{list_index}];
            end
            [fns,pns,~] = uigetfile('*.nii',dstr);
            if fns ~= 0
                sfile = fullfile(pns,fns);
                img_cell{list_index} = sfile;
                set(hObject,'String',img_cell);
            end
        end
    end

    function reset_list(~,~,list)
	%This function resets the content of the list boxes
        reset_string = get(sub_ids,'String');
        if ~isempty(reset_string)
            if ischar(reset_string)
                reset_string = cellstr(reset_string); 
            end
            set(list,'Value',1);
            set(list,'String',reset_string);
        end
    end

    function update_kernel(hObject,~)
        handles = guidata(f_sg);
        handles.kernel = get(hObject,'String');
        guidata(f_sg,handles);
    end

    function browse(~,~)
        folder_name = uigetdir(pwd,'Choose or Create the output folder: ');
        if folder_name ~= 0
            set(output_edit,'String',folder_name)
            handles = guidata(f_sg);
            handles.output_dir = folder_name;
            guidata(f_sg,handles)
        end
    end

    function save_config(~,~)
        [fnsc,pnsc,~] = uiputfile('smooth_config.mat','Save Configuration File: ');
        if fnsc ~= 0
            handles = guidata(f_sg);
            subj_cell = handles.subject_list;
            mask_cell = handles.mask_image_list;
            func_cell = handles.func_image_list;
            thresh = handles.threshold;
            label = handles.label;
            mask_output_dir = handles.mask_output_dir;
            func_output_dir = handles.masked_output_dir;
            kernel = handles.kernel;
            smooth_output_dir = handles.smooth_output_dir;
            filesc = fullfile(pnsc,fnsc);
            save(filesc,'subj_cell','mask_cell','func_cell','thresh','label','mask_output_dir','func_output_dir','kernel','smooth_output_dir');
        end
    end

    function load_config(~,~)
        [fnlc,pnlc,~] = uigetfile('*.mat','Load Configuration File:');
        if fnlc ~= 0
            filelc = fullfile(pnlc,fnlc);
            config = load(filelc);
            handles = guidata(f_sg);
            handles.subject_list = config.subj_cell;
            set(sub_ids,'String',config.subj_cell);
            handles.mask_image_list = config.mask_cell;
            set(mask_list,'String',config.mask_cell);
            handles.func_image_list = config.func_cell;
            set(func_list,'String',config.func_cell);
            handles.threshold = config.thresh;
            handles.label = config.label;
            handles.mask_output_dir = config.mask_output_dir;
            handles.masked_output_dir = config.func_output_dir;
            handles.kernel = config.kernel;
            set(smooth_edit,'String',config.kernel);
            handles.smooth_output_dir = config.smooth_output_dir;
            set(output_edit,'String',config.smooth_output_dir);
            guidata(f_sg,handles);
        end
    end

    function run_smoother(~,~)
        handles = guidata(f_sg);
        subj_cell = handles.subject_list;
        mask_cell = handles.mask_image_list;
        func_cell = handles.func_image_list;
        thresh = handles.threshold;
        if ischar(thresh)
            thresh = str2double(thresh);
        end
        label = handles.label;
        mask_output_dir = handles.mask_output_dir;
        func_output_dir = handles.masked_output_dir;  
        kernel = handles.kernel;
        smooth_output_dir = handles.smooth_output_dir;
        
        c = clock; c_str = [num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_igma_smooth_config_autosave.mat'];
        save(c_str,'subj_cell','mask_cell','func_cell','thresh','label','mask_output_dir','func_output_dir','kernel','smooth_output_dir');
        smooth_programmatic(subj_cell,mask_cell,func_cell,kernel,smooth_output_dir);
        close(f_sg)
    end

end