function [] = smooth_programmatic(subj_cell,mask_cell,func_cell,kernel,output_dir)
%smooth_programmatic calls spm_smooth to smooth the masked functional data.
%It requires the mask data to provide a mask for spm_smooth. This is
%functionally equivalent of spm12's spm_smooth using an implicit mask
%
%Inputs:
%   subj_cell   =   an Nx1 cell array containing the subject id's for N
%                   subjects
%   mask_cell   =   as Nx1 cell array containing the paths to the masks to
%                   provide the implicit mask
%   func_cell   =   an Nx1 cell array containing the paths to the masked
%                   functional images to be smoothed
%   kernel      =   the smoothing kernel. If the kernel is isometric, this
%                   can be a scalar, otherwise it must be a three element
%                   vector structured as [FWHM_x FWHM_y FWHM_z]
%   output_dir  =   the name of the output directory

subs = size(subj_cell,1);
for ss = 1:subs
    if ~exist([output_dir '\' subj_cell{ss}],'dir')
        mkdir([output_dir '\' subj_cell{ss}])
    end
    fV = spm_vol(func_cell{ss});
    P = fV(1).fname;
    Q = [output_dir '\' subj_cell{ss} '\swc4DVolume.nii'];
    s = kernel;
    spm_smooth(P,Q,s);
    
    ffV = spm_vol(Q);
    mfV = ffV;
    mVol = spm_read_vols(spm_vol(mask_cell{ss}));
    for vv = 1:size(ffV,1)
        mfV(vv).fname = [output_dir '\' subj_cell{ss} '\swcMasked_4DVolume.nii'];
        mfV(vv).descript = 'Smoothed, Masked';
        mfVol = spm_read_vols(ffV(vv));
        mfVol = mfVol.*mVol;
        spm_write_vol(mfV(vv),mfVol);
    end
    delete(Q);
    prog = ['Subject ' subj_cell{ss} ' is done.\n'];
    fprintf(prog);
end

end