function switch_to_NaN_gui()
%switch_to_NaN_gui si the gui for switch_to_NaN

if nargin == 0
    subject_list = {};
end

%% GUI Initialization

f_sng = figure('Visible','on','MenuBar','none','Color',[0.94,0.94,0.94], ...
    'Units','normalized','OuterPosition',[0.25,0.2,0.5,0.6]);
uicontrol(f_sng,'Style','text','String','Convert Zero values to NaNs', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0,0.9375,1,0.05]);

p_subj = uipanel(f_sng,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Subject IDs','FontUnits','normalized','FontSize',0.04, ...
    'Units','normalized','Position',[0.025,0.25,(0.95/2),0.7]);
sub_ids = uicontrol(p_subj,'Style','edit','String',subject_list, ...
    'Max',2,'FontUnits','points','FontSize',12, ...
    'Units','normalized','Position',[0.025,0.175,0.95,0.8],'Callback',@update_subj_list);
uicontrol(p_subj,'Style','pushbutton','String','Import Subject List', ...
    'FontUnits','normalized','FontSize',0.35,'Units','normalized', ...
    'Position',[0.0250,0.025,0.4625,0.10],'Callback',@import_subj_list);
uicontrol(p_subj,'Style','pushbutton','String','Export Subject List', ...
    'FontUnits','normalized','FontSize',0.35,'Units','normalized', ...
    'Position',[0.5125,0.025,0.4625,0.10],'Callback',@export_subj_list);

p_func = uipanel(f_sng,'BackgroundColor',[0.9,0.9,0.9], ...
    'Title','Functional Images','FontUnits','normalized', ...
    'FontSize',0.04,'Units','normalized', ...
    'Position',[0.5,0.25,(0.95/2),0.7]);
subj_cell_f = get(sub_ids,'String');
func_list = uicontrol(p_func,'Style','listbox','String',subj_cell_f, ...
    'FontUnits','points','FontSize',12,'Enable','on', ...
    'Units','normalized','Position',[0.025,0.175,0.95,0.8], ...
    'Callback',@get_img_source);
uicontrol(p_func,'Style','pushbutton','String','Reset', ...
    'FontUnits','normalized','FontSize',0.4,'Units','normalized', ...
    'Position',[0.0250,0.025,0.95,0.10],'Callback',{@reset_list,func_list});

uicontrol(f_sng,'Style','text','String','Output Directory:', ...
    'FontUnits','normalized','FontSize',0.55,'Units','normalized', ...
    'Position',[0.025,0.175,0.175,0.05]);
output_edit = uicontrol(f_sng,'Style','edit','FontUnits','normalized', ...
    'FontSize',0.5,'Units','normalized', ...
    'Position',[0.2,0.175,0.65,0.05]);
uicontrol(f_sng,'Style','pushbutton','String','Browse', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.875,0.175,0.1,0.05],'Callback',{@browse,output_edit});

uicontrol(f_sng,'Style','pushbutton','String','Save Config', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.025,0.05,(0.95/3),0.1],'Callback',@save_config);
uicontrol(f_sng,'Style','pushbutton','String','Load Config', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.3417,0.05,(0.95/3),0.1],'Callback',@load_config);
uicontrol(f_sng,'Style','pushbutton','String','Run', ...
    'FontUnits','normalized','FontSize',0.5,'Units','normalized', ...
    'Position',[0.6583,0.05,(0.95/3),0.1],'Callback',@run_switch_to_NaN);

handles = guihandles(f_sng);
subject_list = get(sub_ids,'String');
if ischar(subject_list)
    subject_list = cellstr(subject_list);
end
handles.subject_list = subject_list;
handles.func_image_list = get(func_list,'String');
handles.output_dir = get(output_edit,'String');
guidata(f_sng,handles);

%% Callbacks

    function update_subj_list(hObject,~)
        subj_list = get(hObject,'String');
        if ischar(subj_list)
            subj_list = cellstr(subj_list);
        end
        handles = guidata(f_sng);
        handles.subject_list = subj_list;
        guidata(f_sng,handles);
    end

    function import_subj_list(~,~)
        [fni,pni,~] = uigetfile('*/.mat','Import Subject List:');
        if fni ~=0
            ifile = fullfile(pni,fni);
            list = load(ifile);
            handles = guidata(f_sng);
            handles.subject_list = list.subj_list;
            guidata(f_sng,handles)
            set(sub_ids,'String',list.subj_list);
            set(func_list,'String',list.subj_list);
        end
    end

    function export_subj_list(~,~)
        [subj_list] = get(sub_ids,'String');
        if ischar(subj_list)
            subj_list = cellstr(subj_list);
        end
        handles = guidata(f_sng);
        handles.subject_list = subj_list;
        guidata(f_sng,handles);
        [fne,pne,~] = uiputfile('Subject_List.mat','Export Subject List:');
        if fne ~= 0
            efile = fullfile(pne,fne);
            save(efile,'subj_list');
        end
    end

    function get_img_source(hObject,~)
        list_index = get(hObject,'Value');
        img_cell = get(hObject,'String');
        subj_list = get(sub_ids,'String');
        handles = guidata(f_sng);
        if ~isempty(list_index) && ~isempty(subj_list)
            if ischar(subj_list)
                subj_list = cellstr(subj_list);
            end
            handles.subject_list = subj_list;
            dstr = ['Choose the Functional image for subject ' subj_list{list_index}];
            [fns,pns,~] = uigetfile('*.nii',dstr);
            if fns ~= 0
                sfile = fullfile(pns,fns);
                [~,name,ext] = fileparts(sfile);
                handles.func_image_list{list_index} = sfile;
                guidata(f_sng,handles)
                img_cell{list_index} = [name ext];
                set(hObject,'String',img_cell);
            end
        elseif isempty(list_index) && ~isempty(subj_list)
            if ischar(subj_list)
                subj_list = cellstr(subj_list);
            end
            set(hObject,'Value',1);
            set(hObject,'String',subj_list);
        end
    end

    function reset_list(~,~,list)
        reset_string = get(sub_ids,'String');
        handles = guidata(f_sng);
        if ~isempty(reset_string)
            reset_string = cellstr(reset_string);
        end
        set(list,'Value',1);
        set(list,'String',reset_string);
        handles.func_image_list = get(func_list,'String');
        guidata(f_sng,handles);
    end

    function browse(~,~,edit)
        folder_name = uigetdir(pwd,'Choose or Create the output folder: ');
        if folder_name ~= 0
            set(edit,'String',folder_name);
        end
        handles = guidata(f_sng);
        handles.output_dir = folder_name;
        guidata(f_sng,handles);
    end

    function save_config(~,~)
        [fnsc,pnsc,~] = uiputfile('master_config.mat','Save Configuration File:');
        if fnsc ~= 0
            handle = guidata(f_sng);
            subj_cell = handles.subject_list;
            func_cell = handles.func_image_list;
            output_dir = handles.output_dir;
            filesc = fullfile(pnsc,fnsc);
            save(filesc,'subj_cell','func_cell','output_dir');
        end
    end

    function load_config(~,~)
        [fnlc,pnlc,~] = uigetfile('*.mat','Load Configuration File:');
        if fnls ~= 0
           filelc = fullfile(pnlc,fnlc);
           config = load(filelc);
           handles = guidata(f_sng);
           handles.subject_list = config.subj_cell;
           set(sub_ids,'String',confing.subj_cell);
           handles.func_image_list = config.func_cell;
           set(func_list,'String',config.func_cell);
           handles.output_dir = config.output_dir;
           set(output_edit,'String',config.output_dir);
           guidata(f_sng,handles);
        end
    end

    function run_switch_to_NaN(~,~)
        handles = guidata(f_sng);
        subj_cell = handles.subject_list;
        func_cell = handles.func_image_list;
        output_dir = handles.output_dir;
        fprintf('Converting zeros to NaNs ... ... ... ... ...\n');
        switch_to_NaN(subj_cell,func_cell,output_dir);
        prog_str = ['Done.\NaNed functional images written to ' formatstr(output_dir) '\n'];
        fprintf(prog_str)
        close(f_sng)
    end

end